from __future__ import annotations

import argparse
import dataclasses

from ldaptool._utils import argclasses

from . import _postprocess


def _parser_add_attributes(parser: argparse.ArgumentParser, dest: str) -> None:
    parser.add_argument(
        metavar="attributes",
        dest=dest,
        nargs="*",
        help="""
            Attributes to lookup (and columns to display in tables).
            Fake attributes `dndomain`, `dnpath` an `dnfullpath` are available (created from dn).
        """,
    )


@dataclasses.dataclass(slots=True, kw_only=True)
class Arguments(argclasses.BaseArguments):
    columns: list[str] = dataclasses.field(default_factory=list, metadata=argclasses.manual(_parser_add_attributes))
    columns_keys: list[str] = dataclasses.field(default_factory=list)  # lower case column names
    attributes: list[str] = dataclasses.field(default_factory=list)

    human_separator: str = dataclasses.field(
        default=", ",
        metadata=argclasses.arg(help="Separator to join multiple values of one attribute with (default: %(default)r)"),
    )

    dateonly: bool = dataclasses.field(
        default=True,
        metadata=argclasses.arg(help="Use only date part of decoded timestamps"),
    )

    post_process: dict[str, dict[str, _postprocess.PostProcess]] = dataclasses.field(default_factory=dict)

    def __post_init__(self) -> None:
        super(Arguments, self).__post_init__()  # super() not working here, unclear why.

        # extract special attribute names
        all_attributes = False
        attributes_set: set[str] = set()
        self.columns_keys = []
        for column in list(self.columns):
            column = column.lower()

            if column == "*":
                # '*' not allowed as table column, but for LDIF this means: get ALL attributes + do post processing
                self.columns.remove("*")
                all_attributes = True
                continue

            self.columns_keys.append(column)

            if column == "dndomain":
                self.post_process.setdefault("dn", {})[column] = _postprocess.parse_steps(["domain"])
                attributes_set.add("dn")
            elif column == "dnpath":
                self.post_process.setdefault("dn", {})[column] = _postprocess.parse_steps(["path"])
                attributes_set.add("dn")
            elif column == "dnfullpath":
                self.post_process.setdefault("dn", {})[column] = _postprocess.parse_steps(["fullpath"])
                attributes_set.add("dn")
            else:
                step_names = column.split(":")
                attributes_set.add(step_names[0])
                if len(step_names) > 1:
                    source = step_names.pop(0)
                    self.post_process.setdefault(source, {})[column] = _postprocess.parse_steps(step_names)

        if all_attributes:
            self.attributes = []
        else:
            self.attributes = list(attributes_set)
